@@ -1,8 +1,7 @@
 #!/usr/bin/env bash
 set -e
 
-PYTHON_VERSIONS="${PYTHON_VERSIONS-3.7 3.8 3.9 3.10 3.11}"
-DEPGROUPS="-G async"
+PYTHON_VERSIONS="${PYTHON_VERSIONS-3.8 3.9 3.10 3.11 3.12}"
 
 if ! command -v pdm &>/dev/null; then
     if ! command -v pipx &>/dev/null; then
@@ -11,11 +10,11 @@ if ! command -v pdm &>/dev/null; then
     pipx install pdm
 fi
 if ! pdm self list 2>/dev/null | grep -q pdm-multirun; then
-    pipx inject pdm pdm-multirun
+    pdm install --plugins
 fi
 
 if [ -n "${PYTHON_VERSIONS}" ]; then
-    pdm multirun -vi ${PYTHON_VERSIONS// /,} pdm install ${DEPGROUPS}
+    pdm multirun -vi ${PYTHON_VERSIONS// /,} pdm install -G:all
 else
-    pdm install ${DEPGROUPS}
+    pdm install -G:all
 fi
