@@ -27,13 +27,11 @@ import sys
 from inspect import Parameter as SignatureParameter
 from inspect import Signature, cleandoc, getmodule, ismodule
 from inspect import signature as getsignature
-from pathlib import Path
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from griffe.agents.base import BaseInspector
-from griffe.agents.extensions import Extensions
 from griffe.agents.nodes import ObjectKind, ObjectNode, safe_get_annotation
-from griffe.collections import LinesCollection
+from griffe.collections import LinesCollection, ModulesCollection
 from griffe.dataclasses import (
     Alias,
     Attribute,
@@ -45,14 +43,20 @@ from griffe.dataclasses import (
     ParameterKind,
     Parameters,
 )
-from griffe.docstrings.parsers import Parser
-from griffe.expressions import Expression, Name
+from griffe.extensions import Extensions
 from griffe.importer import dynamic_import
 
+if TYPE_CHECKING:
+    from pathlib import Path
+
+    from griffe.docstrings.parsers import Parser
+    from griffe.expressions import Expression, Name
+
+
 if sys.version_info < (3, 8):
     from cached_property import cached_property
 else:
-    from functools import cached_property  # noqa: WPS440
+    from functools import cached_property
 
 empty = Signature.empty
 
@@ -67,6 +71,7 @@ def inspect(
     docstring_parser: Parser | None = None,
     docstring_options: dict[str, Any] | None = None,
     lines_collection: LinesCollection | None = None,
+    modules_collection: ModulesCollection | None = None,
 ) -> Module:
     """Inspect a module.
 
@@ -79,12 +84,14 @@ def inspect(
         docstring_parser: The docstring parser to use. By default, no parsing is done.
         docstring_options: Additional docstring parsing options.
         lines_collection: A collection of source code lines.
+        modules_collection: A collection of modules.
 
     Returns:
         The module, with its members populated.
     """
-    if not import_paths and filepath:
-        import_paths = [filepath.parent]
+    import_paths = list(import_paths) if import_paths else []
+    if filepath and filepath.parent not in import_paths:
+        import_paths.insert(0, filepath.parent)
     return Inspector(
         module_name,
         filepath,
@@ -93,6 +100,7 @@ def inspect(
         docstring_parser=docstring_parser,
         docstring_options=docstring_options,
         lines_collection=lines_collection,
+        modules_collection=modules_collection,
     ).get_module(import_paths)
 
 
@@ -116,7 +124,10 @@ def _should_create_alias(parent: ObjectNode, child: ObjectNode, current_module_p
     if isinstance(child_obj, cached_property):
         child_obj = child_obj.func
 
-    child_module = getmodule(child_obj)
+    try:
+        child_module = getmodule(child_obj)
+    except Exception:  # noqa: BLE001
+        return None
     if not child_module:
         return None
 
@@ -145,7 +156,7 @@ def _should_create_alias(parent: ObjectNode, child: ObjectNode, current_module_p
     return None
 
 
-class Inspector(BaseInspector):  # noqa: WPS338
+class Inspector(BaseInspector):
     """This class is used to instantiate an inspector.
 
     Inspectors iterate on objects members to extract data from them.
@@ -160,6 +171,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
         docstring_parser: Parser | None = None,
         docstring_options: dict[str, Any] | None = None,
         lines_collection: LinesCollection | None = None,
+        modules_collection: ModulesCollection | None = None,
     ) -> None:
         """Initialize the inspector.
 
@@ -171,6 +183,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
             docstring_parser: The docstring parser to use.
             docstring_options: The docstring parsing options.
             lines_collection: A collection of source code lines.
+            modules_collection: A collection of modules.
         """
         super().__init__()
         self.module_name: str = module_name
@@ -181,10 +194,14 @@ class Inspector(BaseInspector):  # noqa: WPS338
         self.docstring_parser: Parser | None = docstring_parser
         self.docstring_options: dict[str, Any] = docstring_options or {}
         self.lines_collection: LinesCollection = lines_collection or LinesCollection()
+        self.modules_collection: ModulesCollection = modules_collection or ModulesCollection()
 
     def _get_docstring(self, node: ObjectNode) -> Docstring | None:
-        # access `__doc__` directly to avoid taking the `__doc__` attribute from a parent class
-        value = getattr(node.obj, "__doc__", None)
+        try:
+            # access `__doc__` directly to avoid taking the `__doc__` attribute from a parent class
+            value = getattr(node.obj, "__doc__", None)
+        except Exception:  # noqa: BLE001  # getattr can trigger exceptions
+            return None
         if value is None:
             return None
         try:
@@ -232,7 +249,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
         for after_inspector in self.extensions.after_inspection:
             after_inspector.inspect(node)
 
-    def generic_inspect(self, node: ObjectNode) -> None:  # noqa: WPS231
+    def generic_inspect(self, node: ObjectNode) -> None:
         """Extend the base generic inspection with extensions.
 
         Parameters:
@@ -268,6 +285,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
             parent=self.parent,
             docstring=self._get_docstring(node),
             lines_collection=self.lines_collection,
+            modules_collection=self.modules_collection,
         )
         self.generic_inspect(node)
 
@@ -277,7 +295,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
         Parameters:
             node: The node to inspect.
         """
-        bases = [base.__name__ for base in node.obj.__bases__ if base is not object]  # noqa: WPS609
+        bases = [base.__name__ for base in node.obj.__bases__ if base is not object]
 
         class_ = Class(
             name=node.name,
@@ -370,7 +388,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
         """
         self.handle_function(node, {"property"})
 
-    def handle_function(self, node: ObjectNode, labels: set | None = None):  # noqa: WPS231
+    def handle_function(self, node: ObjectNode, labels: set | None = None) -> None:
         """Handle a function.
 
         Parameters:
@@ -379,20 +397,21 @@ class Inspector(BaseInspector):  # noqa: WPS338
         """
         try:
             signature = getsignature(node.obj)
-        except Exception:
+        except Exception:  # noqa: BLE001
             # so many exceptions can be raised here:
             # AttributeError, NameError, RuntimeError, ValueError, TokenError, TypeError
             parameters = None
             returns = None
         else:
             parameters = Parameters(
-                *[_convert_parameter(parameter, parent=self.current) for parameter in signature.parameters.values()]
+                *[_convert_parameter(parameter, parent=self.current) for parameter in signature.parameters.values()],
             )
             return_annotation = signature.return_annotation
-            if return_annotation is empty:
-                returns = None
-            else:
-                returns = _convert_object_to_annotation(return_annotation, parent=self.current)
+            returns = (
+                None
+                if return_annotation is empty
+                else _convert_object_to_annotation(return_annotation, parent=self.current)
+            )
 
         obj: Attribute | Function
         labels = labels or set()
@@ -421,7 +440,7 @@ class Inspector(BaseInspector):  # noqa: WPS338
         """
         self.handle_attribute(node)
 
-    def handle_attribute(self, node: ObjectNode, annotation: str | Name | Expression | None = None):  # noqa: WPS231
+    def handle_attribute(self, node: ObjectNode, annotation: str | Name | Expression | None = None) -> None:
         """Handle an attribute.
 
         Parameters:
@@ -444,16 +463,17 @@ class Inspector(BaseInspector):  # noqa: WPS338
 
         try:
             value = repr(node.obj)
-        except Exception:  # could trigger anything
+        except Exception:  # noqa: BLE001
             value = None
-        docstring = self._get_docstring(node)
+        try:
+            docstring = self._get_docstring(node)
+        except Exception:  # noqa: BLE001
+            docstring = None
 
         attribute = Attribute(
             name=node.name,
             value=value,
             annotation=annotation,
-            # lineno=node.lineno,
-            # endlineno=node.end_lineno,
             docstring=docstring,
         )
         attribute.labels |= labels
@@ -472,28 +492,30 @@ _kind_map = {
 }
 
 
-def _convert_parameter(parameter, parent):
+def _convert_parameter(parameter: SignatureParameter, parent: Module | Class) -> Parameter:
     name = parameter.name
-    if parameter.annotation is empty:
-        annotation = None
-    else:
-        annotation = _convert_object_to_annotation(parameter.annotation, parent=parent)
+    annotation = (
+        None if parameter.annotation is empty else _convert_object_to_annotation(parameter.annotation, parent=parent)
+    )
     kind = _kind_map[parameter.kind]
     if parameter.default is empty:
         default = None
+    elif hasattr(parameter.default, "__name__"):
+        # avoid repr containing chevrons and memory addresses
+        default = parameter.default.__name__
     else:
         default = repr(parameter.default)
     return Parameter(name, annotation=annotation, kind=kind, default=default)
 
 
-def _convert_object_to_annotation(obj, parent):
+def _convert_object_to_annotation(obj: Any, parent: Module | Class) -> str | Name | Expression | None:
     # even when *we* import future annotations,
     # the object from which we get a signature
     # can come from modules which did *not* import them,
     # so inspect.signature returns actual Python objects
     # that we must deal with
     if not isinstance(obj, str):
-        if hasattr(obj, "__name__"):
+        if hasattr(obj, "__name__"):  # noqa: SIM108
             # simple types like int, str, custom classes, etc.
             obj = obj.__name__
         else:
@@ -503,4 +525,4 @@ def _convert_object_to_annotation(obj, parent):
         annotation_node = compile(obj, mode="eval", filename="<>", flags=ast.PyCF_ONLY_AST, optimize=2)
     except SyntaxError:
         return obj
-    return safe_get_annotation(annotation_node.body, parent=parent)
+    return safe_get_annotation(annotation_node.body, parent=parent)  # type: ignore[attr-defined]
