@@ -12,21 +12,20 @@ import ast
 import inspect
 from contextlib import suppress
 from itertools import zip_longest
-from pathlib import Path
-from typing import Any, Iterable
+from typing import TYPE_CHECKING, Any, Iterable
 
 from griffe.agents.base import BaseVisitor
-from griffe.agents.extensions import Extensions
 from griffe.agents.nodes import (
     ASTNode,
-    get_annotation,
     get_docstring,
     get_instance_names,
     get_names,
     get_parameter_default,
-    parse__all__,
     relative_to_absolute,
+    safe_get__all__,
     safe_get_annotation,
+    safe_get_base_class,
+    safe_get_condition,
     safe_get_value,
 )
 from griffe.collections import LinesCollection, ModulesCollection
@@ -43,9 +42,16 @@ from griffe.dataclasses import (
     ParameterKind,
     Parameters,
 )
-from griffe.docstrings.parsers import Parser
-from griffe.exceptions import LastNodeError, NameResolutionError
-from griffe.expressions import Expression, Name
+from griffe.exceptions import AliasResolutionError, CyclicAliasError, LastNodeError, NameResolutionError
+from griffe.expressions import Expression
+from griffe.extensions import Extensions
+
+if TYPE_CHECKING:
+    from pathlib import Path
+
+    from griffe.docstrings.parsers import Parser
+    from griffe.expressions import Name
+
 
 builtin_decorators = {
     "property": "property",
@@ -105,7 +111,7 @@ def visit(
     ).get_module()
 
 
-class Visitor(BaseVisitor):  # noqa: WPS338
+class Visitor(BaseVisitor):
     """This class is used to instantiate a visitor.
 
     Visitors iterate on AST nodes to extract data from them.
@@ -149,7 +155,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         self.modules_collection: ModulesCollection = modules_collection or ModulesCollection()
         self.type_guarded: bool = False
 
-    def _get_docstring(self, node: ast.AST, strict: bool = False) -> Docstring | None:
+    def _get_docstring(self, node: ast.AST, *, strict: bool = False) -> Docstring | None:
         value, lineno, endlineno = get_docstring(node, strict=strict)
         if value is None:
             return None
@@ -187,7 +193,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         for after_visitor in self.extensions.after_visit:
             after_visitor.visit(node)
 
-    def generic_visit(self, node: ast.AST) -> None:  # noqa: WPS231
+    def generic_visit(self, node: ast.AST) -> None:
         """Extend the base generic visit with extensions.
 
         Parameters:
@@ -195,7 +201,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         """
         for before_visitor in self.extensions.before_children_visit:
             before_visitor.visit(node)
-        for child in node.children:  # type: ignore[attr-defined]  # noqa: WPS437
+        for child in node.children:  # type: ignore[attr-defined]
             self.visit(child)
         for after_visitor in self.extensions.after_children_visit:
             after_visitor.visit(node)
@@ -232,7 +238,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                         safe_get_value(decorator_node, self.current.relative_filepath),  # type: ignore[arg-type]
                         lineno=decorator_node.lineno,
                         endlineno=decorator_node.end_lineno,  # type: ignore[attr-defined]
-                    )
+                    ),
                 )
         else:
             lineno = node.lineno
@@ -241,7 +247,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         bases = []
         if node.bases:
             for base in node.bases:
-                bases.append(safe_get_annotation(base, parent=self.current))
+                bases.append(safe_get_base_class(base, parent=self.current))
 
         class_ = Class(
             name=node.name,
@@ -258,7 +264,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         self.generic_visit(node)
         self.current = self.current.parent  # type: ignore[assignment]
 
-    def decorators_to_labels(self, decorators: list[Decorator]) -> set[str]:  # noqa: WPS231
+    def decorators_to_labels(self, decorators: list[Decorator]) -> set[str]:
         """Build and return a set of labels based on decorators.
 
         Parameters:
@@ -304,7 +310,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                     return self.current[base_name], base_function
         return None, None
 
-    def handle_function(self, node: ast.AsyncFunctionDef | ast.FunctionDef, labels: set | None = None):  # noqa: WPS231
+    def handle_function(self, node: ast.AsyncFunctionDef | ast.FunctionDef, labels: set | None = None) -> None:
         """Handle a function definition node.
 
         Parameters:
@@ -320,6 +326,8 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             lineno = node.decorator_list[0].lineno
             for decorator_node in node.decorator_list:
                 decorator_value = safe_get_value(decorator_node, self.filepath)
+                if decorator_value is None:
+                    continue
                 overload = (
                     decorator_value in typing_overload
                     or decorator_value == "overload"
@@ -330,7 +338,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                         decorator_value,  # type: ignore[arg-type]
                         lineno=decorator_node.lineno,
                         endlineno=decorator_node.end_lineno,  # type: ignore[attr-defined]
-                    )
+                    ),
                 )
         else:
             lineno = node.lineno
@@ -366,7 +374,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         # TODO: probably some optimizations to do here
         args_kinds_defaults: Iterable = reversed(
             (
-                *zip_longest(  # noqa: WPS356
+                *zip_longest(
                     reversed(
                         (
                             *zip_longest(
@@ -380,7 +388,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                     reversed(node.args.defaults),
                     fillvalue=None,
                 ),
-            )
+            ),
         )
         arg: ast.arg
         kind: ParameterKind
@@ -398,26 +406,26 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                     annotation=annotation,
                     kind=ParameterKind.var_positional,
                     default="()",
-                )
+                ),
             )
 
         # TODO: probably some optimizations to do here
         kwargs_defaults: Iterable = reversed(
             (
-                *zip_longest(  # noqa: WPS356
+                *zip_longest(
                     reversed(node.args.kwonlyargs),
                     reversed(node.args.kw_defaults),
                     fillvalue=None,
                 ),
-            )
+            ),
         )
         kwarg: ast.arg
         kwarg_default: ast.AST | None
-        for kwarg, kwarg_default in kwargs_defaults:  # noqa: WPS440
+        for kwarg, kwarg_default in kwargs_defaults:
             annotation = safe_get_annotation(kwarg.annotation, parent=self.current)
             default = get_parameter_default(kwarg_default, self.filepath, self.lines_collection)
             parameters.add(
-                Parameter(kwarg.arg, annotation=annotation, kind=ParameterKind.keyword_only, default=default)
+                Parameter(kwarg.arg, annotation=annotation, kind=ParameterKind.keyword_only, default=default),
             )
 
         if node.args.kwarg:
@@ -427,8 +435,8 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                     node.args.kwarg.arg,
                     annotation=annotation,
                     kind=ParameterKind.var_keyword,
-                    default="{}",  # noqa: P103
-                )
+                    default="{}",
+                ),
             )
 
         function = Function(
@@ -456,7 +464,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             self.current[node.name] = function
             if self.current.kind in {Kind.MODULE, Kind.CLASS} and self.current.overloads[function.name]:
                 function.overloads = self.current.overloads[function.name]
-                del self.current.overloads[function.name]  # noqa: WPS420
+                del self.current.overloads[function.name]
 
         function.labels |= labels
 
@@ -499,20 +507,19 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 runtime=not self.type_guarded,
             )
 
-    def visit_importfrom(self, node: ast.ImportFrom) -> None:  # noqa: WPS231
+    def visit_importfrom(self, node: ast.ImportFrom) -> None:
         """Visit an "import from" node.
 
         Parameters:
             node: The node to visit.
         """
         for name in node.names:
-            if not node.module and node.level == 1:
-                if not name.asname:
-                    # special case: when being in `a` and doing `from . import b`,
-                    # we are effectively creating a member `b` in `a` that is pointing to `a.b`
-                    # -> cyclic alias! in that case, we just skip it, as both the member and module
-                    # have the same name and can be accessed the same way
-                    continue
+            if not node.module and node.level == 1 and not name.asname:
+                # special case: when being in `a` and doing `from . import b`,
+                # we are effectively creating a member `b` in `a` that is pointing to `a.b`
+                # -> cyclic alias! in that case, we just skip it, as both the member and module
+                # have the same name and can be accessed the same way
+                continue
 
             alias_path = relative_to_absolute(node, name, self.current.module)
             if name.name == "*":
@@ -529,11 +536,11 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 runtime=not self.type_guarded,
             )
 
-    def handle_attribute(  # noqa: WPS231
+    def handle_attribute(
         self,
         node: ast.Assign | ast.AnnAssign,
         annotation: str | Name | Expression | None = None,
-    ):
+    ) -> None:
         """Handle an attribute (assignment) node.
 
         Parameters:
@@ -554,7 +561,19 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 names = get_names(node)
             except KeyError:  # unsupported nodes, like subscript
                 return
-            labels.add("class-attribute")
+
+            if isinstance(annotation, Expression) and annotation.is_classvar:
+                # explicit classvar: class attribute only
+                annotation = annotation[2]
+                labels.add("class-attribute")
+            elif node.value:
+                # attribute assigned at class-level: available in instances as well
+                labels.add("class-attribute")
+                labels.add("instance-attribute")
+            else:
+                # annotated attribute only: not available at class-level
+                labels.add("instance-attribute")
+
         elif parent.kind is Kind.FUNCTION:
             if parent.name != "__init__":
                 return
@@ -587,6 +606,16 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 if isinstance(node.parent, (ast.If, ast.ExceptHandler)):  # type: ignore[union-attr]
                     continue  # prefer "no-exception" case
 
+                existing_member = parent.members[name]
+                with suppress(AliasResolutionError, CyclicAliasError):
+                    labels |= existing_member.labels  # type: ignore[misc]
+                    # forward previous docstring and annotation instead of erasing them
+                    if existing_member.docstring and not docstring:
+                        docstring = existing_member.docstring
+                    with suppress(AttributeError):
+                        if existing_member.annotation and not annotation:  # type: ignore[union-attr]
+                            annotation = existing_member.annotation  # type: ignore[union-attr]
+
             attribute = Attribute(
                 name=name,
                 value=value,
@@ -601,7 +630,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
 
             if name == "__all__":
                 with suppress(AttributeError):
-                    parent.exports = parse__all__(node, self.current)  # type: ignore[assignment,arg-type]
+                    parent.exports = safe_get__all__(node, self.current)  # type: ignore[assignment,arg-type]
 
     def visit_assign(self, node: ast.Assign) -> None:
         """Visit an assignment node.
@@ -633,7 +662,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             )
             if all_augment:
                 # we assume exports is not None at this point
-                self.current.exports.extend(parse__all__(node, self.current))  # type: ignore[arg-type,union-attr]
+                self.current.exports.extend(safe_get__all__(node, self.current))  # type: ignore[arg-type,union-attr]
 
     def visit_if(self, node: ast.If) -> None:
         """Visit an "if" node.
@@ -642,10 +671,9 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             node: The node to visit.
         """
         if isinstance(node.parent, (ast.Module, ast.ClassDef)):  # type: ignore[attr-defined]
-            with suppress(KeyError):  # unhandled AST nodes
-                condition = get_annotation(node.test, parent=self.current)
-                if str(condition) in {"typing.TYPE_CHECKING", "TYPE_CHECKING"}:
-                    self.type_guarded = True
+            condition = safe_get_condition(node.test, parent=self.current, log_level=None)
+            if str(condition) in {"typing.TYPE_CHECKING", "TYPE_CHECKING"}:
+                self.type_guarded = True
         self.generic_visit(node)
         self.type_guarded = False
 
@@ -655,11 +683,10 @@ _patched = False
 
 def patch_ast() -> None:
     """Extend the base `ast.AST` class to provide more functionality."""
-    global _patched  # noqa: WPS420
+    global _patched  # noqa: PLW0603
     if _patched:
         return
     for name, member in inspect.getmembers(ast):
-        if name != "AST" and inspect.isclass(member):
-            if ast.AST in member.__bases__:  # noqa: WPS609
-                member.__bases__ = (*member.__bases__, ASTNode)  # noqa: WPS609
-    _patched = True  # noqa: WPS122,WPS442
+        if name != "AST" and inspect.isclass(member) and ast.AST in member.__bases__:
+            member.__bases__ = (*member.__bases__, ASTNode)
+    _patched = True
