@@ -13,20 +13,23 @@ from collections import defaultdict
 from contextlib import suppress
 from pathlib import Path
 from textwrap import dedent
-from typing import Any, Callable, Union, cast
+from typing import TYPE_CHECKING, Any, Callable, Union, cast
 
-from griffe.collections import LinesCollection, ModulesCollection
-from griffe.docstrings.dataclasses import DocstringSection
-from griffe.docstrings.parsers import Parser, parse  # noqa: WPS347
+from griffe.docstrings.parsers import Parser, parse
 from griffe.exceptions import AliasResolutionError, BuiltinModuleError, CyclicAliasError, NameResolutionError
-from griffe.expressions import Expression, Name
 from griffe.mixins import GetMembersMixin, ObjectAliasMixin, SerializationMixin, SetMembersMixin
 
+if TYPE_CHECKING:
+    from griffe.collections import LinesCollection, ModulesCollection
+    from griffe.docstrings.dataclasses import DocstringSection
+    from griffe.expressions import Expression, Name
+
+
 # TODO: remove once Python 3.7 support is dropped
 if sys.version_info < (3, 8):
     from cached_property import cached_property
 else:
-    from functools import cached_property  # noqa: WPS440
+    from functools import cached_property
 
 
 class ParameterKind(enum.Enum):
@@ -67,7 +70,7 @@ class Decorator:
         self.lineno: int | None = lineno
         self.endlineno: int | None = endlineno
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # noqa: ARG002
         """Return this decorator's data as a dictionary.
 
         Parameters:
@@ -154,7 +157,7 @@ class Docstring:
         """
         return parse(self, parser or self.parser, **(options or self.parser_options))
 
-    def as_dict(self, full: bool = False, docstring_parser: Parser | None = None, **kwargs: Any) -> dict[str, Any]:
+    def as_dict(self, *, full: bool = False, docstring_parser: Parser | None = None, **kwargs: Any) -> dict[str, Any]:
         """Return this docstring's data as a dictionary.
 
         Parameters:
@@ -221,7 +224,7 @@ class Parameter:
         """
         return self.default is None
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # noqa: ARG002
         """Return this parameter's data as a dictionary.
 
         Parameters:
@@ -302,7 +305,7 @@ class Kind(enum.Enum):
     ALIAS: str = "alias"
 
 
-class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMixin):  # noqa: WPS215
+class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMixin):
     """An abstract class representing a Python object.
 
     Attributes:
@@ -374,19 +377,16 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
     @property
     def has_docstring(self) -> bool:
         """Tell if this object has a non-empty docstring."""
-        return bool(self.docstring)  # noqa: DAR201
+        return bool(self.docstring)
 
     @property
     def has_docstrings(self) -> bool:
         """Tell if this object or any of its members has a non-empty docstring."""
-        if self.has_docstring:  # noqa: DAR201
+        if self.has_docstring:
             return True
-        for member in self.members.values():
-            if member.has_docstrings:  # type: ignore[union-attr]
-                return True
-        return False
+        return any(member.has_docstrings for member in self.members.values())
 
-    def member_is_exported(self, member: Object | Alias, explicitely: bool = True) -> bool:
+    def member_is_exported(self, member: Object | Alias, *, explicitely: bool = True) -> bool:
         """Tell if a member of this object is "exported".
 
         By exported, we mean that the object is included in the `__all__` attribute
@@ -423,7 +423,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
         if isinstance(kind, set):
             if not kind:
                 raise ValueError("kind must not be an empty set")
-            return self.kind in (knd if isinstance(knd, Kind) else Kind(knd) for knd in kind)  # noqa: WPS509,WPS510
+            return self.kind in (knd if isinstance(knd, Kind) else Kind(knd) for knd in kind)
         if isinstance(kind, str):
             kind = Kind(kind)
         return self.kind is kind
@@ -431,22 +431,22 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
     @property
     def is_module(self) -> bool:
         """Tell if this object is a module."""
-        return self.kind is Kind.MODULE  # noqa: DAR201
+        return self.kind is Kind.MODULE
 
     @property
     def is_class(self) -> bool:
         """Tell if this object is a class."""
-        return self.kind is Kind.CLASS  # noqa: DAR201
+        return self.kind is Kind.CLASS
 
     @property
     def is_function(self) -> bool:
         """Tell if this object is a function."""
-        return self.kind is Kind.FUNCTION  # noqa: DAR201
+        return self.kind is Kind.FUNCTION
 
     @property
     def is_attribute(self) -> bool:
         """Tell if this object is an attribute."""
-        return self.kind is Kind.ATTRIBUTE  # noqa: DAR201
+        return self.kind is Kind.ATTRIBUTE
 
     def has_labels(self, labels: set[str]) -> bool:
         """Tell if this object has all the given labels.
@@ -526,7 +526,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
             return self
         if self.parent is not None:
             return self.parent.module
-        raise ValueError
+        raise ValueError(f"Object {self.name} does not have a parent module")
 
     @cached_property
     def package(self) -> Module:
@@ -549,8 +549,8 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
         """
         return self.module.filepath
 
-    @cached_property  # noqa: WPS231
-    def relative_filepath(self) -> Path:  # noqa: WPS231
+    @cached_property
+    def relative_package_filepath(self) -> Path:
         """Return the file path where this object was defined, relative to the top module path.
 
         Raises:
@@ -560,23 +560,46 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
             A file path.
         """
         package_path = self.package.filepath
-        if isinstance(self.module.filepath, list):
+        if isinstance(self.filepath, list):
             if isinstance(package_path, list):
                 for pkg_path in package_path:
-                    for self_path in self.module.filepath:
+                    for self_path in self.filepath:
                         with suppress(ValueError):
                             return self_path.relative_to(pkg_path.parent)
             else:
-                for self_path in self.module.filepath:  # noqa: WPS440
+                for self_path in self.filepath:
                     with suppress(ValueError):
                         return self_path.relative_to(package_path.parent.parent)
             raise ValueError
         if isinstance(package_path, list):
-            for pkg_path in package_path:  # noqa: WPS440
+            for pkg_path in package_path:
                 with suppress(ValueError):
-                    return self.module.filepath.relative_to(pkg_path.parent)
+                    return self.filepath.relative_to(pkg_path.parent)
             raise ValueError
-        return self.module.filepath.relative_to(package_path.parent.parent)
+        return self.filepath.relative_to(package_path.parent.parent)
+
+    @cached_property
+    def relative_filepath(self) -> Path:
+        """Return the file path where this object was defined, relative to the current working directory.
+
+        If this object's file path is not relative to the current working directory, return its absolute path.
+
+        Raises:
+            ValueError: When the relative path could not be computed.
+
+        Returns:
+            A file path.
+        """
+        cwd = Path.cwd()
+        if isinstance(self.filepath, list):
+            for self_path in self.filepath:
+                with suppress(ValueError):
+                    return self_path.relative_to(cwd)
+            raise ValueError(f"No directory in {self.filepath!r} is relative to the current working directory {cwd}")
+        try:
+            return self.filepath.relative_to(cwd)
+        except ValueError:
+            return self.filepath
 
     @cached_property
     def path(self) -> str:
@@ -694,7 +717,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
             return self.parent.path
         return self.parent.resolve(name)
 
-    def as_dict(self, full: bool = False, **kwargs: Any) -> dict[str, Any]:
+    def as_dict(self, *, full: bool = False, **kwargs: Any) -> dict[str, Any]:
         """Return this object's data as a dictionary.
 
         Parameters:
@@ -715,7 +738,8 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
                     "path": self.path,
                     "filepath": self.filepath,
                     "relative_filepath": self.relative_filepath,
-                }
+                    "relative_package_filepath": self.relative_package_filepath,
+                },
             )
 
         if self.lineno:
@@ -749,7 +773,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin, SerializationMi
         return blockfinder.last
 
 
-class Alias(ObjectAliasMixin):  # noqa: WPS338
+class Alias(ObjectAliasMixin):
     """This class represents an alias, or indirection, to an object declared in another module.
 
     Aliases represent objects that are in the scope of a module or class,
@@ -812,11 +836,11 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
     def __repr__(self) -> str:
         return f"<Alias({self.name!r}, {self.target_path!r})>"
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str | tuple[str, ...]):
         # not handled by __getattr__
         return self.target[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str | tuple[str, ...], value: Object | Alias):
         # not handled by __getattr__
         self.target[key] = value
 
@@ -834,7 +858,7 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         """
         # custom behavior to avoid raising exceptions
         try:
-            return self.target.kind
+            return self.final_target.kind
         except (AliasResolutionError, CyclicAliasError):
             return Kind.ALIAS
 
@@ -845,9 +869,7 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         Returns:
             The target lineno or the alias lineno.
         """
-        if self.resolved:
-            return self.target.lineno
-        return self.alias_lineno
+        return self.final_target.lineno
 
     @property
     def endlineno(self) -> int | None:
@@ -856,19 +878,23 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         Returns:
             The target endlineno or the alias endlineno.
         """
-        if self.resolved:
-            return self.target.endlineno
-        return self.alias_endlineno
+        return self.final_target.endlineno
 
     @property
     def has_docstring(self) -> bool:
         """Tell if this alias' target has a non-empty docstring."""
-        return self.resolved and self.target.has_docstring  # noqa: DAR201
+        try:
+            return self.final_target.has_docstring
+        except (AliasResolutionError, CyclicAliasError):
+            return False
 
     @property
     def has_docstrings(self) -> bool:
         """Tell if this alias' target or any of its members has a non-empty docstring."""
-        return self.resolved and self.target.has_docstrings  # noqa: DAR201
+        try:
+            return self.final_target.has_docstrings
+        except (AliasResolutionError, CyclicAliasError):
+            return False
 
     @property
     def parent(self) -> Module | Class | None:
@@ -906,113 +932,117 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
     # GENERIC OBJECT PROXIES --------------------------------
 
     @property
-    def docstring(self):  # noqa: D102
-        return self.target.docstring
+    def docstring(self) -> Docstring | None:  # noqa: D102
+        return self.final_target.docstring
+
+    @docstring.setter
+    def docstring(self, docstring: Docstring | None) -> None:
+        self.final_target.docstring = docstring
 
     @property
-    def members(self):  # noqa: D102
-        return self.target.members
+    def members(self) -> dict[str, Object | Alias]:  # noqa: D102
+        return self.final_target.members
 
     @property
-    def labels(self):  # noqa: D102
-        return self.target.labels
+    def labels(self) -> set[str]:  # noqa: D102
+        return self.final_target.labels
 
     @property
-    def imports(self):  # noqa: D102
-        return self.target.imports
+    def imports(self) -> dict[str, str]:  # noqa: D102
+        return self.final_target.imports
 
     @property
-    def exports(self):  # noqa: D102
-        return self.target.exports
+    def exports(self) -> set[str] | list[str | Name] | None:  # noqa: D102
+        return self.final_target.exports
 
     @property
-    def aliases(self):  # noqa: D102
-        return self.target.aliases
+    def aliases(self) -> dict[str, Alias]:  # noqa: D102
+        return self.final_target.aliases
 
-    def member_is_exported(self, member: Object | Alias, explicitely: bool = True) -> bool:  # noqa: D102
-        return self.target.member_is_exported(member, explicitely)
+    def member_is_exported(self, member: Object | Alias, *, explicitely: bool = True) -> bool:  # noqa: D102
+        return self.final_target.member_is_exported(member, explicitely=explicitely)
 
     def is_kind(self, kind: str | Kind | set[str | Kind]) -> bool:  # noqa: D102
-        return self.target.is_kind(kind)
+        return self.final_target.is_kind(kind)
 
     @property
-    def is_module(self):  # noqa: D102
-        return self.target.is_module
+    def is_module(self) -> bool:  # noqa: D102
+        return self.final_target.is_module
 
     @property
-    def is_class(self):  # noqa: D102
-        return self.target.is_class
+    def is_class(self) -> bool:  # noqa: D102
+        return self.final_target.is_class
 
     @property
-    def is_function(self):  # noqa: D102
-        return self.target.is_function
+    def is_function(self) -> bool:  # noqa: D102
+        return self.final_target.is_function
 
     @property
-    def is_attribute(self):  # noqa: D102
-        return self.target.is_attribute
+    def is_attribute(self) -> bool:  # noqa: D102
+        return self.final_target.is_attribute
 
     def has_labels(self, labels: set[str]) -> bool:  # noqa: D102
-        return self.target.has_labels(labels)
+        return self.final_target.has_labels(labels)
 
     def filter_members(self, *predicates: Callable[[Object | Alias], bool]) -> dict[str, Object | Alias]:  # noqa: D102
-        return self.target.filter_members(*predicates)
+        return self.final_target.filter_members(*predicates)
 
     @property
-    def modules(self):  # noqa: D102
-        return self.target.modules
+    def modules(self) -> dict[str, Module]:  # noqa: D102
+        return self.final_target.modules
 
     @property
-    def classes(self):  # noqa: D102
-        return self.target.classes
+    def classes(self) -> dict[str, Class]:  # noqa: D102
+        return self.final_target.classes
 
     @property
-    def functions(self):  # noqa: D102
-        return self.target.functions
+    def functions(self) -> dict[str, Function]:  # noqa: D102
+        return self.final_target.functions
 
     @property
-    def attributes(self):  # noqa: D102
-        return self.target.attributes
+    def attributes(self) -> dict[str, Attribute]:  # noqa: D102
+        return self.final_target.attributes
 
     @property
-    def module(self):  # noqa: D102
-        return self.target.module
+    def module(self) -> Module:  # noqa: D102
+        return self.final_target.module
 
     @property
-    def package(self):  # noqa: D102
-        return self.target.package
+    def package(self) -> Module:  # noqa: D102
+        return self.final_target.package
 
     @property
-    def filepath(self):  # noqa: D102
-        return self.target.filepath
+    def filepath(self) -> Path | list[Path]:  # noqa: D102
+        return self.final_target.filepath
 
     @property
-    def relative_filepath(self):  # noqa: D102
-        return self.target.relative_filepath
+    def relative_filepath(self) -> Path:  # noqa: D102
+        return self.final_target.relative_filepath
 
     @property
-    def canonical_path(self):  # noqa: D102
-        return self.target.canonical_path
+    def canonical_path(self) -> str:  # noqa: D102
+        return self.final_target.canonical_path
 
     @property
-    def lines_collection(self):  # noqa: D102
-        return self.target.lines_collection
+    def lines_collection(self) -> LinesCollection:  # noqa: D102
+        return self.final_target.lines_collection
 
     @property
-    def lines(self):  # noqa: D102
-        return self.target.lines
+    def lines(self) -> list[str]:  # noqa: D102
+        return self.final_target.lines
 
     @property
-    def source(self):  # noqa: D102
-        return self.target.source
+    def source(self) -> str:  # noqa: D102
+        return self.final_target.source
 
     def resolve(self, name: str) -> str:  # noqa: D102
-        return self.target.resolve(name)
+        return self.final_target.resolve(name)
 
     # SPECIFIC MODULE/CLASS/FUNCTION/ATTRIBUTE PROXIES ---------------
 
     @property
-    def _filepath(self) -> Path | list[Path] | None:  # noqa: D102
-        return cast(Module, self.target)._filepath  # noqa: WPS437
+    def _filepath(self) -> Path | list[Path] | None:
+        return cast(Module, self.target)._filepath
 
     @property
     def bases(self) -> list[Name | Expression | str]:  # noqa: D102
@@ -1087,7 +1117,26 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         if self.parent is not None:
             self._target.aliases[self.path] = self
 
-    def resolve_target(self) -> None:  # noqa: WPS231,WPS238
+    @property
+    def final_target(self) -> Object:
+        """Resolve and return the final target, if possible.
+
+        This will iterate through the targets until a non-alias object is found.
+
+        Returns:
+            The final target.
+        """
+        # using a dict as an ordered set
+        paths_seen: dict[str, None] = {}
+        target = self
+        while target.is_alias:
+            if target.path in paths_seen:
+                raise CyclicAliasError([*paths_seen, target.path])
+            paths_seen[target.path] = None
+            target = target.target  # type: ignore[assignment,union-attr]
+        return target  # type: ignore[return-value]
+
+    def resolve_target(self) -> None:
         """Resolve the target.
 
         Raises:
@@ -1101,7 +1150,7 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         if self._passed_through:
             raise CyclicAliasError([self.target_path])
         self._passed_through = True
-        try:  # noqa: WPS501
+        try:
             self._resolve_target()
         finally:
             self._passed_through = False
@@ -1110,14 +1159,14 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         try:
             resolved = self.modules_collection[self.target_path]
         except KeyError as error:
-            raise AliasResolutionError(self.target_path) from error
+            raise AliasResolutionError(self) from error
         if resolved is self:
             raise CyclicAliasError([self.target_path])
         if resolved.is_alias and not resolved.resolved:
             try:
                 resolved.resolve_target()
-            except CyclicAliasError as error:  # noqa: WPS440
-                raise CyclicAliasError([self.target_path] + error.chain)
+            except CyclicAliasError as error:
+                raise CyclicAliasError([self.target_path, *error.chain]) from error
         self._target = resolved
         if self.parent is not None:
             self._target.aliases[self.path] = self  # type: ignore[union-attr]  # we just set the target
@@ -1146,7 +1195,7 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
             return self.target_path
         return None
 
-    def as_dict(self, full: bool = False, **kwargs: Any) -> dict[str, Any]:
+    def as_dict(self, *, full: bool = False, **kwargs: Any) -> dict[str, Any]:  # noqa: ARG002
         """Return this alias' data as a dictionary.
 
         Parameters:
@@ -1165,10 +1214,10 @@ class Alias(ObjectAliasMixin):  # noqa: WPS338
         if full:
             base["path"] = self.path
 
-        if self.lineno:
-            base["lineno"] = self.lineno
-        if self.endlineno:
-            base["endlineno"] = self.endlineno
+        if self.alias_lineno:
+            base["lineno"] = self.alias_lineno
+        if self.alias_endlineno:
+            base["endlineno"] = self.alias_endlineno
 
         return base
 
@@ -1210,6 +1259,19 @@ class Module(Object):
             raise BuiltinModuleError(self.name)
         return self._filepath
 
+    @cached_property
+    def imports_future_annotations(self) -> bool:
+        """Tell whether this module import future annotations.
+
+        Returns:
+            True or false.
+        """
+        return (
+            "annotations" in self.members
+            and self.members["annotations"].is_alias
+            and self.members["annotations"].target_path == "__future__.annotations"  # type: ignore[union-attr]
+        )
+
     @cached_property
     def is_init_module(self) -> bool:
         """Tell if this module is an `__init__.py` module.
@@ -1326,7 +1388,7 @@ class Class(Object):
                     *[
                         Parameter(attr.name, annotation=attr.annotation, default=attr.value)
                         for attr in self.attributes.values()
-                    ]
+                    ],
                 )
             return Parameters()
 
