@@ -70,10 +70,17 @@ from griffe.expressions import Expression, Name
 from griffe.logger import LogLevel
 
 if TYPE_CHECKING:
-    from typing import Any, Literal, Pattern  # type: ignore[attr-defined]
+    import sys
+    from typing import Any, Pattern
 
     from griffe.dataclasses import Docstring
 
+    if sys.version_info < (3, 8):
+        from typing_extensions import Literal
+    else:
+        from typing import Literal
+
+
 _warn = warning(__name__)
 
 _section_kind = {
@@ -90,15 +97,15 @@ _section_kind = {
 }
 
 
-def _is_empty_line(line) -> bool:
+def _is_empty_line(line: str) -> bool:
     return not line.strip()
 
 
-def _is_dash_line(line) -> bool:
+def _is_dash_line(line: str) -> bool:
     return not _is_empty_line(line) and _is_empty_line(line.replace("-", ""))
 
 
-def _read_block_items(docstring: Docstring, offset: int) -> tuple[list[list[str]], int]:  # noqa: WPS231,WPS234
+def _read_block_items(docstring: Docstring, *, offset: int) -> tuple[list[list[str]], int]:
     lines = docstring.lines
     if offset >= len(lines):
         return [], offset
@@ -158,7 +165,7 @@ def _read_block_items(docstring: Docstring, offset: int) -> tuple[list[list[str]
     return items, new_offset - 1
 
 
-def _read_block(docstring: Docstring, offset: int) -> tuple[str, int]:  # noqa: WPS231
+def _read_block(docstring: Docstring, *, offset: int) -> tuple[str, int]:
     lines = docstring.lines
     if offset >= len(lines):
         return "", offset
@@ -186,10 +193,21 @@ def _read_block(docstring: Docstring, offset: int) -> tuple[str, int]:  # noqa:
 _RE_OB: str = r"\{"  # opening bracket
 _RE_CB: str = r"\}"  # closing bracket
 _RE_NAME: str = r"\*{0,2}[_a-z][_a-z0-9]*"
-_RE_TYPE: str = r"[_a-z0-9 \[\]|().,'\"-]*"
-_RE_RETURNS: Pattern = re.compile(rf"^(?:(?P<name>{_RE_NAME})\s*:\s*)?(?P<type>{_RE_TYPE})", re.IGNORECASE)
+_RE_TYPE: str = r".+"
+_RE_RETURNS: Pattern = re.compile(
+    rf"""
+    (?:
+        (?P<nt_name>{_RE_NAME})\s*:\s*(?P<nt_type>{_RE_TYPE})  # name and type
+        |  # or
+        (?P<name>{_RE_NAME})\s*:\s*  # just name
+        |  # or
+        (?P<type>{_RE_TYPE})\s*  # just type
+    )
+    """,
+    re.IGNORECASE | re.VERBOSE,
+)
 _RE_YIELDS: Pattern = _RE_RETURNS
-_RE_RECEIVES: Pattern = _RE_YIELDS
+_RE_RECEIVES: Pattern = _RE_RETURNS
 _RE_PARAMETER: Pattern = re.compile(
     rf"""
     (?P<names>{_RE_NAME}(?:,\s{_RE_NAME})*)
@@ -197,15 +215,8 @@ _RE_PARAMETER: Pattern = re.compile(
         \s:\s
         (?:
             (?:{_RE_OB}(?P<choices>.+){_RE_CB})|
-            (?:
-                (?P<type>{_RE_TYPE})
-                (?:,\soptional)?
-                (?:
-                    ,\sdefault\s*[:=]\s*
-                    (?P<default>.+)
-                )?
-            )
-        )
+            (?P<type>{_RE_TYPE})
+        )?
     )?
     """,
     re.IGNORECASE | re.VERBOSE,
@@ -214,15 +225,16 @@ _RE_DOCTEST_BLANKLINE: Pattern = re.compile(r"^\s*<BLANKLINE>\s*$")
 _RE_DOCTEST_FLAGS: Pattern = re.compile(r"(\s*#\s*doctest:.+)$")
 
 
-def _read_parameters(  # noqa: WPS231
+def _read_parameters(
     docstring: Docstring,
+    *,
     offset: int,
     warn_unknown_params: bool = True,
 ) -> tuple[list[DocstringParameter], int]:
     parameters = []
     annotation: str | Name | Expression | None
 
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     for item in items:
         match = _RE_PARAMETER.match(item[0])
@@ -231,18 +243,20 @@ def _read_parameters(  # noqa: WPS231
             continue
 
         names = match.group("names").split(", ")
-        annotation = match.group("type")
-        annotation = annotation or None
+        annotation = match.group("type") or None
         choices = match.group("choices")
+        default = None
         if choices:
             choices = choices.split(", ", 1)
             default = choices[0]
-        else:
-            default = match.group("default")
-        if len(item) > 1:
-            description = "\n".join(item[1:]).rstrip()
-        else:
-            description = ""
+        elif annotation:
+            match = re.match(r"^(?P<annotation>.+),\s+default(?: |: |=)(?P<default>.+)$", annotation)
+            if match:
+                default = match.group("default")
+                annotation = match.group("annotation")
+        if annotation and annotation.endswith(", optional"):
+            annotation = annotation[:-10]
+        description = "\n".join(item[1:]).rstrip() if len(item) > 1 else ""
 
         if annotation is None:
             # try to use the annotation from the signature
@@ -256,7 +270,7 @@ def _read_parameters(  # noqa: WPS231
             annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
 
         if default is None:
-            for name in names:  # noqa: WPS440
+            for name in names:
                 with suppress(AttributeError, KeyError):
                     default = docstring.parent.parameters[name].default  # type: ignore[union-attr]
                     break
@@ -264,7 +278,7 @@ def _read_parameters(  # noqa: WPS231
         if warn_unknown_params:
             with suppress(AttributeError):  # for parameters sections in objects without parameters
                 params = docstring.parent.parameters  # type: ignore[union-attr]
-                for name in names:  # noqa: WPS440
+                for name in names:
                     if name not in params:
                         message = f"Parameter '{name}' does not appear in the function signature"
                         for starred_name in (f"*{name}", f"**{name}"):
@@ -273,7 +287,7 @@ def _read_parameters(  # noqa: WPS231
                                 break
                         _warn(docstring, new_offset, message)
 
-        for name in names:  # noqa: WPS440
+        for name in names:
             parameters.append(DocstringParameter(name, value=default, annotation=annotation, description=description))
 
     return parameters, new_offset
@@ -281,10 +295,11 @@ def _read_parameters(  # noqa: WPS231
 
 def _read_parameters_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionParameters | None, int]:
-    parameters, new_offset = _read_parameters(docstring, offset)
+    parameters, new_offset = _read_parameters(docstring, offset=offset)
 
     if parameters:
         return DocstringSectionParameters(parameters), new_offset
@@ -295,10 +310,11 @@ def _read_parameters_section(
 
 def _read_other_parameters_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionOtherParameters | None, int]:
-    parameters, new_offset = _read_parameters(docstring, offset, warn_unknown_params=False)
+    parameters, new_offset = _read_parameters(docstring, offset=offset, warn_unknown_params=False)
 
     if parameters:
         return DocstringSectionOtherParameters(parameters), new_offset
@@ -309,13 +325,14 @@ def _read_other_parameters_section(
 
 def _read_deprecated_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionDeprecated | None, int]:
     # deprecated
     # SINCE_VERSION
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty deprecated section at line {offset}")
@@ -330,15 +347,15 @@ def _read_deprecated_section(
     return DocstringSectionDeprecated(version=version, text=text), new_offset
 
 
-def _read_returns_section(  # noqa: WPS231
+def _read_returns_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionReturns | None, int]:
-    # returns
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty returns section at line {offset}")
@@ -351,8 +368,9 @@ def _read_returns_section(  # noqa: WPS231
             _warn(docstring, new_offset, f"Could not parse line '{item[0]}'")
             continue
 
-        name, annotation = match.groups()
-        annotation = annotation or None
+        groups = match.groupdict()
+        name = groups["nt_name"] or groups["name"]
+        annotation = groups["nt_type"] or groups["type"]
         text = dedent("\n".join(item[1:]))
         if annotation is None:
             # try to retrieve the annotation from the docstring parent
@@ -385,15 +403,16 @@ def _read_returns_section(  # noqa: WPS231
     return DocstringSectionReturns(returns), new_offset
 
 
-def _read_yields_section(  # noqa: WPS231
+def _read_yields_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionYields | None, int]:
     # yields
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty yields section at line {offset}")
@@ -406,41 +425,42 @@ def _read_yields_section(  # noqa: WPS231
             _warn(docstring, new_offset, f"Could not parse line '{item[0]}'")
             continue
 
-        name, annotation = match.groups()
-        annotation = annotation or None
+        groups = match.groupdict()
+        name = groups["nt_name"] or groups["name"]
+        annotation = groups["nt_type"] or groups["type"]
         text = dedent("\n".join(item[1:]))
         if annotation is None:
             # try to retrieve the annotation from the docstring parent
             with suppress(AttributeError, KeyError, ValueError):
                 annotation = docstring.parent.returns  # type: ignore[union-attr]
-                if len(items) > 1:
-                    if annotation.is_iterator:
-                        yield_item = annotation.iterator_item()
-                    elif annotation.is_generator:
-                        yield_item, _, _ = annotation.generator_items()
-                    else:
-                        raise ValueError
-                    if isinstance(yield_item, Name):
-                        annotation = yield_item
-                    elif yield_item.is_tuple:
-                        annotation = yield_item.tuple_item(index)
-                    else:
-                        annotation = yield_item
+                if annotation.is_iterator:
+                    yield_item = annotation.iterator_item()
+                elif annotation.is_generator:
+                    yield_item, _, _ = annotation.generator_items()
+                else:
+                    raise ValueError
+                if isinstance(yield_item, Name):
+                    annotation = yield_item
+                elif yield_item.is_tuple:
+                    annotation = yield_item.tuple_item(index)
+                else:
+                    annotation = yield_item
         else:
             annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
         yields.append(DocstringYield(name=name or "", annotation=annotation, description=text))
     return DocstringSectionYields(yields), new_offset
 
 
-def _read_receives_section(  # noqa: WPS231
+def _read_receives_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionReceives | None, int]:
     # receives
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty receives section at line {offset}")
@@ -453,14 +473,15 @@ def _read_receives_section(  # noqa: WPS231
             _warn(docstring, new_offset, f"Could not parse line '{item[0]}'")
             continue
 
-        name, annotation = match.groups()
-        annotation = annotation or None
+        groups = match.groupdict()
+        name = groups["nt_name"] or groups["name"]
+        annotation = groups["nt_type"] or groups["type"]
         text = dedent("\n".join(item[1:]))
         if annotation is None:
             # try to retrieve the annotation from the docstring parent
             with suppress(AttributeError, KeyError):
                 annotation = docstring.parent.returns  # type: ignore[union-attr]
-                if len(items) > 1 and annotation.is_generator:
+                if annotation.is_generator:
                     _, receives_item, _ = annotation.generator_items()
                     if isinstance(receives_item, Name):
                         annotation = receives_item
@@ -476,13 +497,14 @@ def _read_receives_section(  # noqa: WPS231
 
 def _read_raises_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionRaises | None, int]:
     # raises
     # EXCEPTION
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty raises section at line {offset}")
@@ -498,13 +520,14 @@ def _read_raises_section(
 
 def _read_warns_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionWarns | None, int]:
     # warns
     # WARNING
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty warns section at line {offset}")
@@ -520,13 +543,14 @@ def _read_warns_section(
 
 def _read_attributes_section(
     docstring: Docstring,
+    *,
     offset: int,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionAttributes | None, int]:
     # attributes (for classes)
     # NAME( : TYPE)?
     #    TEXT?
-    items, new_offset = _read_block_items(docstring, offset)
+    items, new_offset = _read_block_items(docstring, offset=offset)
 
     if not items:
         _warn(docstring, new_offset, f"Empty attributes section at line {offset}")
@@ -553,13 +577,14 @@ def _read_attributes_section(
     return DocstringSectionAttributes(attributes), new_offset
 
 
-def _read_examples_section(  # noqa: WPS231
+def _read_examples_section(
     docstring: Docstring,
+    *,
     offset: int,
     trim_doctest_flags: bool = True,
-    **options: Any,
+    **options: Any,  # noqa: ARG001
 ) -> tuple[DocstringSectionExamples | None, int]:
-    text, new_offset = _read_block(docstring, offset)
+    text, new_offset = _read_block(docstring, offset=offset)
 
     sub_sections: list[tuple[Literal[DocstringSectionKind.text] | Literal[DocstringSectionKind.examples], str]] = []
     in_code_example = False
@@ -579,12 +604,12 @@ def _read_examples_section(  # noqa: WPS231
 
         elif in_code_example:
             if trim_doctest_flags:
-                line = _RE_DOCTEST_FLAGS.sub("", line)
-                line = _RE_DOCTEST_BLANKLINE.sub("", line)
+                line = _RE_DOCTEST_FLAGS.sub("", line)  # noqa: PLW2901
+                line = _RE_DOCTEST_BLANKLINE.sub("", line)  # noqa: PLW2901
             current_example.append(line)
 
         elif line.startswith("```"):
-            in_code_block = not in_code_block  # noqa: WPS434
+            in_code_block = not in_code_block
             current_text.append(line)
 
         elif in_code_block:
@@ -597,7 +622,7 @@ def _read_examples_section(  # noqa: WPS231
             in_code_example = True
 
             if trim_doctest_flags:
-                line = _RE_DOCTEST_FLAGS.sub("", line)
+                line = _RE_DOCTEST_FLAGS.sub("", line)  # noqa: PLW2901
             current_example.append(line)
 
         else:
@@ -629,8 +654,9 @@ _section_reader = {
 }
 
 
-def parse(  # noqa: WPS231
+def parse(
     docstring: Docstring,
+    *,
     ignore_init_summary: bool = False,
     trim_doctest_flags: bool = True,
     **options: Any,
@@ -662,7 +688,7 @@ def parse(  # noqa: WPS231
     }
 
     ignore_summary = (
-        options["ignore_init_summary"]  # noqa: WPS222
+        options["ignore_init_summary"]
         and docstring.parent is not None
         and docstring.parent.name == "__init__"
         and docstring.parent.is_function
@@ -670,10 +696,7 @@ def parse(  # noqa: WPS231
         and docstring.parent.parent.is_class
     )
 
-    if ignore_summary:
-        offset = 2
-    else:
-        offset = 0
+    offset = 2 if ignore_summary else 0
 
     while offset < len(lines):
         line_lower = lines[offset].lower()
@@ -689,7 +712,7 @@ def parse(  # noqa: WPS231
                     sections.append(DocstringSectionText("\n".join(current_section).rstrip("\n")))
                 current_section = []
             reader = _section_reader[_section_kind[line_lower]]
-            section, offset = reader(docstring, offset + 2, **options)  # type: ignore[operator]
+            section, offset = reader(docstring, offset=offset + 2, **options)  # type: ignore[operator]
             if section:
                 sections.append(section)
 
