@@ -7,11 +7,11 @@ from contextlib import suppress
 from typing import TYPE_CHECKING, Callable
 
 from griffe.agents.nodes import safe_get_annotation
-from griffe.expressions import Expression, Name
 from griffe.logger import LogLevel, get_logger
 
 if TYPE_CHECKING:
     from griffe.dataclasses import Docstring
+    from griffe.expressions import Expression, Name
 
 
 def warning(name: str) -> Callable[[Docstring, int, str], None]:
@@ -32,7 +32,7 @@ def warning(name: str) -> Callable[[Docstring, int, str], None]:
     """
     logger = get_logger(name)
 
-    def warn(docstring: Docstring, offset: int, message: str) -> None:  # noqa: WPS430
+    def warn(docstring: Docstring, offset: int, message: str) -> None:
         try:
             prefix = docstring.parent.relative_filepath  # type: ignore[union-attr]
         except (AttributeError, ValueError):
@@ -63,9 +63,9 @@ def parse_annotation(
         SyntaxError,  # annotation contains syntax errors
     ):
         code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
-        if code.body:
+        if code.body:  # type: ignore[attr-defined]
             name_or_expr = safe_get_annotation(
-                code.body,
+                code.body,  # type: ignore[attr-defined]
                 parent=docstring.parent,  # type: ignore[arg-type]
                 log_level=log_level,
             )
